@@ -1,8 +1,28 @@
 import ql2_pb2 as p
 import types
 import sys
+from threading import Lock
 from errors import *
-from net import Connection
+import repl # For the repl connection
+
+# This is both an external function and one used extensively
+# internally to convert coerce python values to RQL types
+def expr(val):
+    '''
+        Convert a Python primitive into a RQL primitive value
+    '''
+    if isinstance(val, RqlQuery):
+        return val
+    elif isinstance(val, list):
+        return MakeArray(*val)
+    elif isinstance(val, dict):
+        # MakeObj doesn't take the dict as a keyword args to avoid
+        # conflicting with the `self` parameter.
+        return MakeObj(val)
+    elif callable(val):
+        return Func(val)
+    else:
+        return Datum(val)
 
 class RqlQuery(object):
 
@@ -19,8 +39,8 @@ class RqlQuery(object):
     # Send this query to the server to be executed
     def run(self, c=None, **global_opt_args):
         if not c:
-            if Connection.repl_connection:
-                c = Connection.repl_connection
+            if repl.default_connection:
+                c = repl.default_connection
             else:
                 raise RqlDriverError("RqlQuery.run must be given a connection to run on.")
 
@@ -221,14 +241,8 @@ class RqlQuery(object):
     def order_by(self, *obs):
         return OrderBy(self, *obs)
 
-    def between(self, left_bound=None, right_bound=None):
-        # This is odd and inconsistent with the rest of the API. Blame a
-        # poorly thought out spec.
-        if left_bound == None:
-            left_bound = ()
-        if right_bound == None:
-            right_bound = ()
-        return Between(self, left_bound=left_bound, right_bound=right_bound)
+    def between(self, left_bound=None, right_bound=None, index=()):
+        return Between(self, left_bound, right_bound, index=index)
 
     def distinct(self):
         return Distinct(self)
@@ -247,8 +261,8 @@ class RqlQuery(object):
     def outer_join(self, other, predicate):
         return OuterJoin(self, other, predicate)
 
-    def eq_join(self, left_attr, other):
-        return EqJoin(self, left_attr, other)
+    def eq_join(self, left_attr, other, index=()):
+        return EqJoin(self, left_attr, other, index=index)
 
     def zip(self):
         return Zip(self)
@@ -264,6 +278,9 @@ class RqlQuery(object):
     def for_each(self, mapping):
         return ForEach(self, func_wrap(mapping))
 
+    def info(self):
+        return Info(self)
+
 # These classes define how nodes are printed by overloading `compose`
 
 def needs_wrap(arg):
@@ -278,7 +295,7 @@ class RqlBiOperQuery(RqlQuery):
 
 class RqlTopLevelQuery(RqlQuery):
     def compose(self, args, optargs):
-        args.extend([repr(name)+'='+optargs[name] for name in optargs.keys()])
+        args.extend([name+'='+optargs[name] for name in optargs.keys()])
         return T('r.', self.st, '(', T(*(args), intsp=', '), ')')
 
 class RqlMethodQuery(RqlQuery):
@@ -333,7 +350,15 @@ class Datum(RqlQuery):
         elif datum.type == p.Datum.R_BOOL:
             return datum.r_bool
         elif datum.type == p.Datum.R_NUM:
-            return datum.r_num
+            # Convert to an integer if we think maybe the user might think of this
+            # number as an integer. I have been assured that this is a "temporary"
+            # behavior change until RQL supports native integers.
+            num = datum.r_num
+            if num % 1 == 0:
+                # Then we assume that in the user's data model this floating point
+                # number is meant be an integer and "helpfully" convert types for them.
+                num = int(num)
+            return num
         elif datum.type == p.Datum.R_STR:
             return datum.r_str
         elif datum.type == p.Datum.R_ARRAY:
@@ -366,6 +391,8 @@ class MakeObj(RqlQuery):
 
         self.optargs = {}
         for k in obj_dict.keys():
+            if not isinstance(k, types.StringTypes):
+                raise RqlDriverError("Object keys must be strings.");
             self.optargs[k] = expr(obj_dict[k])
 
     def compose(self, args, optargs):
@@ -494,13 +521,13 @@ class DB(RqlTopLevelQuery):
     def table_list(self):
         return TableList(self)
 
-    def table_create(self, table_name, primary_key=(), datacenter=(), cache_size=()):
-        return TableCreate(self, table_name, primary_key=primary_key, datacenter=datacenter, cache_size=cache_size)
+    def table_create(self, table_name, primary_key=(), datacenter=(), cache_size=(), hard_durability=()):
+        return TableCreate(self, table_name, primary_key=primary_key, datacenter=datacenter, cache_size=cache_size, hard_durability=hard_durability)
 
     def table_drop(self, table_name):
         return TableDrop(self, table_name)
 
-    def table(self, table_name, use_outdated=False):
+    def table(self, table_name, use_outdated=()):
         return Table(self, table_name, use_outdated=use_outdated)
 
 class FunCall(RqlQuery):
@@ -525,6 +552,21 @@ class Table(RqlQuery):
     def get(self, key):
         return Get(self, key)
 
+    def get_all(self, key, index=()):
+        return GetAll(self, key, index=index)
+
+    def index_create(self, name, fundef=None):
+        if fundef:
+            return IndexCreate(self, name, func_wrap(fundef))
+        else:
+            return IndexCreate(self, name)
+
+    def index_drop(self, name):
+        return IndexDrop(self, name)
+
+    def index_list(self):
+        return IndexList(self)
+
     def compose(self, args, optargs):
         if isinstance(self.args[0], DB):
             return T(args[0], '.table(', args[1], ')')
@@ -535,6 +577,10 @@ class Get(RqlMethodQuery):
     tt = p.Term.GET
     st = 'get'
 
+class GetAll(RqlMethodQuery):
+    tt = p.Term.GET_ALL
+    st = 'get_all'
+
 class Reduce(RqlMethodQuery):
     tt = p.Term.REDUCE
     st = 'reduce'
@@ -645,6 +691,18 @@ class TableList(RqlMethodQuery):
     tt = p.Term.TABLE_LIST
     st = "table_list"
 
+class IndexCreate(RqlMethodQuery):
+    tt = p.Term.INDEX_CREATE
+    st = 'index_create'
+
+class IndexDrop(RqlMethodQuery):
+    tt = p.Term.INDEX_DROP
+    st = 'index_drop'
+
+class IndexList(RqlMethodQuery):
+    tt = p.Term.INDEX_LIST
+    st = 'index_list'
+
 class Branch(RqlTopLevelQuery):
     tt = p.Term.BRANCH
     st = "branch"
@@ -658,9 +716,13 @@ class All(RqlBiOperQuery):
     st = "&"
 
 class ForEach(RqlMethodQuery):
-    tt =p.Term.FOREACH
+    tt = p.Term.FOREACH
     st = 'for_each'
 
+class Info(RqlMethodQuery):
+    tt = p.Term.INFO
+    st = 'info'
+
 # Called on arguments that should be functions
 def func_wrap(val):
     val = expr(val)
@@ -685,6 +747,7 @@ def func_wrap(val):
 
 class Func(RqlQuery):
     tt = p.Term.FUNC
+    lock = Lock()
     nextVarId = 1
 
     def __init__(self, lmbd):
@@ -693,7 +756,9 @@ class Func(RqlQuery):
         for i in xrange(lmbd.func_code.co_argcount):
             vrs.append(Var(Func.nextVarId))
             vrids.append(Func.nextVarId)
+            Func.lock.acquire()
             Func.nextVarId += 1
+            Func.lock.release()
 
         self.vrs = vrs
         self.args = [MakeArray(*vrids), expr(lmbd(*vrs))]
@@ -709,5 +774,3 @@ class Asc(RqlTopLevelQuery):
 class Desc(RqlTopLevelQuery):
     tt = p.Term.DESC
     st = 'desc'
-
-from query import expr
